@@ -13,7 +13,7 @@ our @EXPORT_OK = (
 );
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v0.21.4";
+our $VERSION = "v0.22.0";
 
 1;
 __END__
@@ -100,7 +100,7 @@ RxPerl - an implementation of Reactive Extensions / rxjs for Perl
 This module is an implementation of L<Reactive Extensions|http://reactivex.io/> in Perl. It replicates the
 behavior of L<rxjs 6|https://www.npmjs.com/package/rxjs> which is the JavaScript implementation of ReactiveX.
 
-Currently 26 of the 100+ operators in rxjs are implemented in this module.
+Currently 36 of the 100+ operators in rxjs are implemented in this module.
 
 =head1 EXPORTABLE FUNCTIONS
 
@@ -304,6 +304,17 @@ too).
 
 =over
 
+=item op_concat_map
+
+L<https://rxjs.dev/api/operators/concatMap>
+
+    # 0, 1, 2, 0, 1, 2, 0, 1, 2, complete
+    rx_of(10, 20, 30)->pipe(
+        op_concat_map(sub {
+            rx_interval(1)->pipe(op_take(3)),
+        }),
+    )->subscribe($observer);
+
 =item op_delay
 
 L<https://rxjs.dev/api/operators/delay>
@@ -348,6 +359,17 @@ L<https://rxjs.dev/api/operators/distinctUntilKeyChanged>
         op_distinct_until_key_changed('name'),
     )->subscribe($observer);
 
+=item op_exhaust_map
+
+L<https://rxjs.dev/api/operators/exhaustMap>
+
+    # 0, 1, 2, complete
+    rx_of(10, 20, 30)->pipe(
+        op_exhaust_map(sub {
+            rx_interval(1)->pipe( op_take(3) );
+        }),
+    )->subscribe($observer);
+
 =item op_filter
 
 L<https://rxjs.dev/api/operators/filter>
