@@ -21,28 +21,24 @@ our %EXPORT_TAGS = (all => \@EXPORT_OK);
 sub rx_observable;
 
 sub _rx_concat_helper {
-    my ($sources, $subscriber, $early_returns) = @_;
+    my ($sources, $subscriber, $active) = @_;
 
-    @$sources or do {
+    if (! @$sources) {
         $subscriber->{complete}->() if defined $subscriber->{complete};
         return;
-    };
+    }
 
     my $source = shift @$sources;
-
     my $own_subscription = RxPerl::Subscription->new;
-    @$early_returns = ($own_subscription);
-    get_subscription_from_subscriber($subscriber)->add_dependents($early_returns);
-
     my $own_subscriber = {
         new_subscription => $own_subscription,
-        next             => $subscriber->{next},
-        error            => $subscriber->{error},
-        complete         => sub {
-            _rx_concat_helper->($sources, $subscriber, $early_returns);
+        next     => $subscriber->{next},
+        error    => $subscriber->{error},
+        complete => sub {
+            _rx_concat_helper($sources, $subscriber, $active);
         },
     };
-
+    @$active = ($own_subscription);
     $source->subscribe($own_subscriber);
 }
 
@@ -54,9 +50,12 @@ sub rx_concat {
 
         my @sources = @sources;
 
-        my $early_returns = [];
-        get_subscription_from_subscriber($subscriber)->add_dependents($early_returns, sub { @sources = () });
-        _rx_concat_helper(\@sources, $subscriber, $early_returns);
+        my @active;
+        get_subscription_from_subscriber($subscriber)->add_dependents(
+            \@active, sub { undef @sources },
+        );
+
+        _rx_concat_helper(\@sources, $subscriber, \@active);
 
         return;
     });
