@@ -12,9 +12,9 @@ use Scalar::Util 'reftype', 'refaddr';
 
 use Exporter 'import';
 our @EXPORT_OK = qw/
-    op_delay op_distinct_until_changed op_distinct_until_key_changed op_filter op_first op_map op_map_to op_merge_map
-    op_multicast op_pairwise op_ref_count op_scan op_share op_start_with op_switch_map op_take op_take_until
-    op_take_while op_tap
+    op_concat_map op_delay op_distinct_until_changed op_distinct_until_key_changed op_exhaust_map op_filter op_first
+    op_map op_map_to op_merge_map op_multicast op_pairwise op_ref_count op_scan op_share op_start_with op_switch_map
+    op_take op_take_until op_take_while op_tap
 /;
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
@@ -22,6 +22,80 @@ our %EXPORT_TAGS = (all => \@EXPORT_OK);
 # Two bugs: 1) script doesn't exit upon the subscriber receiving complete, and 2) delaying of(1, 2, 3) often
 # shows fewer than 3 'next' values and not in the right order.
 
+
+sub _op_concat_map_helper {
+    my (
+        $value, $inner_subscriptions, $observable_factory,
+        $subscriber, $queue, $own_subscription
+    ) = @_;
+
+    if (@$inner_subscriptions and ! $inner_subscriptions->[0]{closed}) {
+        push @$queue, $value;
+        return;
+    }
+
+    my $inner_observable = $observable_factory->($value);
+
+    @$inner_subscriptions = (RxPerl::Subscription->new);
+    $inner_observable->subscribe({
+        new_subscription => $inner_subscriptions->[0],
+        next     => $subscriber->{next},
+        error    => $subscriber->{error},
+        complete => sub {
+            @$inner_subscriptions = ();
+            if (@$queue) {
+                my $new_value = shift @$queue;
+                _op_concat_map_helper(
+                    $new_value, $inner_subscriptions, $observable_factory,
+                    $subscriber, $queue, $own_subscription,
+                );
+            } else {
+                if ($own_subscription->{closed}) {
+                    $subscriber->{complete}->() if defined $subscriber->{complete};
+                }
+            }
+        },
+    });
+}
+
+sub op_concat_map {
+    my ($observable_factory) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return rx_observable->new(sub {
+            my ($subscriber) = @_;
+
+            my @inner_subscriptions;
+            my @queue;
+
+            my $own_subscription = RxPerl::Subscription->new;
+            my $own_subscriber = {
+                new_subscription => $own_subscription,
+                next             => sub {
+                    my ($value) = @_;
+
+                    _op_concat_map_helper(
+                        $value, \@inner_subscriptions, $observable_factory,
+                        $subscriber, \@queue, $own_subscription
+                    );
+                },
+                error            => $subscriber->{error},
+                complete         => sub {
+                    if (! @inner_subscriptions) {
+                        $subscriber->{complete}->() if defined $subscriber->{complete};
+                    }
+                },
+            };
+
+            $source->subscribe($own_subscriber);
+
+            return ($own_subscription, \@inner_subscriptions);
+        });
+    };
+}
+
 sub op_delay {
     my ($delay) = @_;
 
@@ -124,6 +198,53 @@ sub op_distinct_until_key_changed {
     }),
 }
 
+sub op_exhaust_map {
+    my ($observable_factory) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return rx_observable->new(sub {
+            my ($subscriber) = @_;
+
+            my $inner_subscription;
+            my $own_subscription = RxPerl::Subscription->new;
+            my $own_observer = {
+                new_subscription => $own_subscription,
+                next             => sub {
+                    my ($value) = @_;
+
+                    if (!$inner_subscription or $inner_subscription->{closed}) {
+                        my $inner_observable = $observable_factory->($value);
+                        $inner_subscription = RxPerl::Subscription->new;
+                        my $inner_observer = {
+                            new_subscription => $inner_subscription,
+                            next             => $subscriber->{next},
+                            error            => $subscriber->{error},
+                            complete         => sub {
+                                if ($own_subscription->{closed}) {
+                                    $subscriber->{complete}->() if defined $subscriber->{complete};
+                                }
+                            },
+                        };
+                        $inner_observable->subscribe($inner_observer);
+                    }
+                },
+                error            => $subscriber->{error},
+                complete         => sub {
+                    if (! $inner_subscription or $inner_subscription->{closed}) {
+                        $subscriber->{complete}->() if defined $subscriber->{complete};
+                    }
+                },
+            };
+
+            $source->subscribe($own_observer);
+
+            return ($own_subscription, \$inner_subscription);
+        });
+    };
+}
+
 sub op_filter {
     my ($filtering_sub) = @_;
 
